/* -*- mode: C; c-basic-offset: 8; indent-tabs-mode: nil; tab-width: 8 -*- */
/* This program and all accompanying files are copyright 1998 Zachary
 * Beane <xach@xach.com>. They come with NO WARRANTY; see the file COPYING
 * for details.
 * 
 * This program is licensed to you under the terms of the GNU General
 * Public License. You should have recieved a copy of it with this
 * program; if you did not, you can view the terms at http://www.gnu.org/.
 *
 */

#include "config.h"

#include <stdio.h>
#include <stdlib.h>
#include <pwd.h>
#include <string.h>
#include "options.h"
#include "llist.h"
#include "url.h"
#include "http.h"
#include "ftp.h"
#include "gopher.h"
#include "util.h"

#define LATEST_URL "http://www.xach.com/snarf/snarf-latest.tar.gz"

void
usage(void)
{
        printf("This is snarf, version %s\n", VERSION);
        printf("usage: snarf [OPTIONS] URL [OUTFILE] ...\n");
        printf("Options:\n");
        printf("    -a     Force active FTP (default is passive)\n");
        printf("    -v     Verbose; print anything the server " 
                "sends\n");
        printf("    -q     Don't print progress bars (default is "
                "%s)\n",
#ifdef PROGRESS_DEFAULT_OFF
               "off"
#else
               "on"
#endif
               );
        printf("    -p     Force printing of progress bars (overrides -q)\n");
        printf("    -r     Resume downloading a partially "
                "transferred file\n");
        printf("    -m     Spoof MSIE user-agent string\n");
        printf("    -z     Spoof Navigator user-agent string\n");

        printf("\n");

        printf("Lowercase option letters only affect the URLs that "
                "immediately follow them.\n");
        printf("If you give an option in caps, it will be the "
                "default option for all URLs\nthat follow it.\n");

        printf("\n");

        printf("If you specify the outfile as '-', the file will be "
                "printed to standard\noutput as it downloads.\n");

        printf("\n");

        printf("You can have as many URLs and outfiles as you "
                "want\n");

        printf("\n");

        printf("You can specify a username and password for ftp or "
                "http authentication. The\nformat is:\n");
        printf("\n");
        printf("    ftp://username:password@host/\n");
        printf("\n");
        printf("If you don't specify a password, you will be prompted "
                "for one.\n");

        printf("\n");

        printf("snarf checks the SNARF_PROXY, FTP_PROXY, GOPHER_PROXY, "
               "HTTP_PROXY, and PROXY\nenvironment variables.\n");

        printf("\n");

        printf("snarf is free software and has NO WARRANTY. See the file "
               "COPYING for details.\n");

        exit(1);
}
                

int
main(int argc, char *argv[])
{
        List *arglist		= NULL;
        ListNode *node		= NULL;
        UrlResource *rsrc	= NULL;
        Url *u			= NULL;
        int retval		= 0;
        int i;

        arglist = list_new();
        rsrc 	= url_resource_new();
        node 	= list_node_new();

        if(argc < 2) {
                usage(); /* usage() exits */
        }

        for(i = 1; --argc; i++) {
                if( strcmp(argv[i], "LATEST") == 0 ) {
                        u = url_new();
                        if( !url_init(u, LATEST_URL) ) {
                                error("`%s' is not a valid URL");
                                continue;
                        }

                        rsrc->url = u;
                        continue;
                }

                if( ((!rsrc->url) && (argv[i][0] != '-')) ||
                    is_probably_an_url(argv[i]) ) {
                        debug("[%s] is an URL", argv[i]);
                        u = url_new();
                        if( !(url_init(u, argv[i])) ) {
                                error("`%s' is not a valid URL", argv[i]);
                                continue;
                        }
                                
                        /* only happens if is_probably_an_url() */
                        if( rsrc->url ) {
                                debug("adding [%p] to arglist", rsrc);
                                node->data = (void *)rsrc;
                                list_add_node(arglist, node);

                                node = list_node_new();

                                rsrc = url_resource_new();
                        } 

                        rsrc->url = u;

                } else {
                        /* either an output file or an option switch */
                        if( argv[i][0] == '-' && argv[i][1] == '\0' ) {
                                if( rsrc->url )
                                        rsrc->outfile = strdup(argv[i]);
                                else
                                        warn("output file specified " 
                                             "before url");

                                continue;
                        }

                        if( argv[i][0] == '-' ) {
                                if( rsrc->url ) {
                                        node->data = (void *)rsrc;
                                        list_add_node(arglist, node);
                                        
                                        node = list_node_new();

                                        rsrc = url_resource_new();
                                }

                                rsrc->options = set_options(rsrc->options,
                                                            argv[i]);
                        } else {
                                if( rsrc->url ) {
                                        if( !rsrc->outfile )
                                                rsrc->outfile = strdup(argv[i]);
                                        else
                                                warn("multiple output files "
                                                     "specified, ignoring "
                                                     "`%s' for %s", argv[i],
                                                     rsrc->url->full_url);
                                }
                        }
                }       
        }

        if( rsrc->url ) {
                debug("adding [%p] to arglist", rsrc);
                node->data = rsrc;
                list_add_node(arglist, node);
        }

        debug("arglist has [%d] items", arglist->count);

        if( arglist->count < 1 )
                usage();

        /* walk through the arglist and output and do that thing you
           do so well */

        for( node = arglist->first_node; 
             node && node->data; 
             node = node->next ) {
                rsrc = node->data;
                u = rsrc->url;

                if( !rsrc->outfile )
                        if( u->file )
                                rsrc->outfile = strdup(u->file);

                if( rsrc->outfile ) {
                        rsrc->outfile_offset = get_file_size(rsrc->outfile);
                        
                        if( rsrc->outfile_offset && 
                            !(rsrc->options & OPT_NORESUME) ) {
                                fprintf(stderr, "setting resume on "
                                        "existing file `%s'\n", rsrc->outfile);
                                rsrc->options |= OPT_RESUME;
                        }
                }
                        

                if( !u->service_type ) {
                        warn("unknown service in url: `%s'", 
                             u->full_url);
                } else {
                        if( u->username && !u->password ) {
                                int len = 0;
                                char *prompt = NULL;
                        
                                len = strlen("Password for@: ");
                                len += strlen(u->host);
                                len += strlen(u->username);

                                prompt = malloc(len + 1);

                                sprintf(prompt, "Password for %s@%s: ",
                                        u->username, u->host);
                        
                                u->password =  (char *)strdup((char *)
                                                              getpass(prompt));
                                free(prompt);
                        }

                        switch (u->service_type) {
                        case SERVICE_HTTP:
                                i = http_transfer(rsrc);
                                if( i == 0 )
                                        retval++;
                                break;
                        case SERVICE_FTP:
                                i = ftp_transfer(rsrc);
                                if( i == 0 )
                                        retval++;
                                break;
                        case SERVICE_GOPHER:
                                i = gopher_transfer(rsrc);
                                if( i == 0 )
                                        retval++;
                                break;
                        default:
                                error("bad url: %s", u->full_url);
                        }
                }
                
                url_resource_destroy(rsrc);
        }
                
        return retval;
}
        
